package com.caimei365.order.mapper;

import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;


/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/9
 */
@Mapper
public interface BaseMapper {
    /**
     * 供应商名称
     * @param shopId 供应商Id
     */
    String getShopNameById(Integer shopId);
    /**
     * 供应商名称
     * @param clubId 供应商Id
     */
    String getClubNameById(Integer clubId);
    /**
     * 获取商品供应商Id
     * @param productId 商品Id
     */
    Integer getShopIdByproductId(Integer productId);
    /**
     * 获取机构用户Id
     * @param clubId 机构Id
     */
    Integer getUserIdByClubId(Integer clubId);
    /**
     * 根据用户Id查询用户身份
     */
    Integer getIdentityByUserId(Integer userId);
    /**
     * 根据用户Id查询公司名
     */
    String getUserNameByUserId(Integer userId);
    /**
     * 根据用户Id查询手机号
     */
    String getBindMobileByUserId(Integer userId);
    /**
     * 根据商品ID查询阶梯价列表
     */
    List<LadderPriceVo> getLadderPriceList(Integer productId);
    /**
     * 根据商品ID和用户ID 查询复购价
     */
    Double getRepurchasePrice(@Param("productId") Integer productId, @Param("userId") Integer userId);
    /**
     * 供应商促销优惠活动
     * @param shopId 供应商Id
     */
    PromotionsVo getPromotionByShopId(Integer shopId);
    /**
     * 商品促销优惠活动
     * @param productId 商品Id
     */
    PromotionsVo getPromotionByProductId(Integer productId);
    /**
     * 促销优惠活动赠品列表
     * @param promotionsId 促销Id
     */
    List<CartItemVo> getPromotionGifts(Integer promotionsId);
    /**
     * 根据商品Id获取供应商
     * @param productId 商品Id
     */
    CartShopVo getShopByProductId(Integer productId);
    /**
     * 发票信息
     * @param userId 用户Id
     */
    InvoiceVo getUserInvoice(Integer userId);
    /**
     * 根据用户Id获取用户余额
     * @param userId 用户Id
     */
    Double getUserMoney(Integer userId);
    /**
     * 可用余额
     * @param userId 用户Id
     */
    Double getAbleUserMoney(Integer userId);
    /**
     * 可用采美豆
     * @param userId 用户Id
     */
    Integer getUserBeans(Integer userId);
    /**
     * 获取商品运费 （0包邮 1到付 2默认（遵循运费规则））
     * @param productIds 商品Id列表
     */
    List<ProductPostageVo> getPostageFlagList(@Param("productIds") List<String> productIds);
    /**
     * 统计用户订单数
     * @param userId 用户Id
     */
    Integer countUserOrder(Integer userId);
    /**
     * 获取根据地区Id获取省市地址信息
     * @param townId 地区Id
     */
    AddressVo getProvinceIdAndCityId(Integer townId);
    /**
     * 获取详细地址信息
     * @param addressId 地址Id
     */
    Integer getTownIdByAddressId(Integer addressId);
    /**
     * 更新用户增值税发票
     */
    void updateUserInvoiceByUserId(InvoiceVo invoice);
    /**
     * 新增用户增值税发票
     */
    void insertUserInvoiceByUserId(InvoiceVo userInvoice);
    /**
     * 查询当前商品复购价信息
     * @param userId    机构用户Id
     * @param productId 商品Id
     */
    PurchasePricePo getPurchasePricePo(Integer userId, Integer productId);
    /**
     * 更新复购价格库
     */
    void updatePurchasePrice(PurchasePricePo purchase);
    /**
     * 新增复购价格库
     */
    void insertPurchasePrice(PurchasePricePo purchase);
    /**
     * 新增历史复购价记录
     */
    void insertPurchaseHistory(PurchaseHistoryPo purchaseHistory);
    /**
     * 保存 收款记录
     */
    void insertDiscernReceipt(DiscernReceiptPo discernReceipt);
    /**
     * 保存 收款项和订单关系
     */
    void insertOrderReceiptRelation(OrderReceiptRelationPo relation);
    /**
     * 保存 采美豆使用记录
     */
    void insertBeansHistory(UserBeansHistoryPo beansHistory);
    /**
     * 更新用户剩余采美豆数量
     * @param userBeans 采美豆数量
     * @param userId 机构用户Id
     */
    void updateUserBeans(Integer userId, int userBeans);
    /**
     * 保存余额到余额收支记录
     */
    void insertBalanceRecord(BalanceRecordPo balanceRecord);
    /**
     * 获取条款列表
     */
    List<ClauseVo> getClauseList();
    /**
     * 查询超级会员用户id
     * @param userId    用户id
     * @return
     */
    Integer getSvipUserIdByUserId(Integer userId);

    /**
     * 查询超级会员用户id
     * @param clubId    机构id
     * @return
     */
    Integer getSvipUserIdByClubId(Integer clubId);
}
