package com.caimei365.order.mapper;

import com.caimei365.order.model.dto.CartDto;
import com.caimei365.order.model.dto.InvoiceDto;
import com.caimei365.order.model.po.CartPo;
import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/25
 */
@Mapper
public interface CartClubMapper {
    /**
     * 购物车供应商列表
     * @param userId 用户Id
     */
    List<CartShopVo> getCartShops(Integer userId);
    /**
     * 购物车供应商下商品列表
     * @param shopId 供应商Id
     * @param userId 用户Id
     */
    List<CartItemVo> getCartProductsByShopId(@Param("shopId") Integer shopId, @Param("userId") Integer userId);
    /**
     * 获取购物车商品列表（不区分供应商）
     * @param userId 用户Id
     */
    List<CartItemVo> getCartProductList(Integer userId);
    /**
     * 获取购物车
     * @param cartDto userId,productId
     */
    CartPo getCartPo(CartDto cartDto);
    /**
     * 更新购物车
     * @param cart CartPo
     */
    void updateCart(CartPo cart);
    /**
     * 新增购物车
     * @param cart CartPo
     */
    void insertCart(CartPo cart);
    /**
     * 删除购物车
     * @param userId     用户Id
     * @param productIds 商品Id集合
     */
    void deleteCartByProductIds(@Param("userId") Integer userId, @Param("productIds") List<String> productIds);
    /**
     * 根据商品Ids获取购物车供应商列表
     * @param userId 用户Id
     */
    List<CartShopVo> getCartShopsByProductIds(@Param("userId") Integer userId, @Param("productIds") List<String> productIds);
    /**
     * 根据商品Ids获取购物车已上架商品列表
     * @param userId 用户Id
     */
    List<CartItemVo> getCartProductsByShopIdAndProductIds(@Param("userId") Integer userId,@Param("shopId")  Integer shopId, @Param("productIds") List<String> productIds);
    /**
     * 删除用户发票信息
     */
    void deleteUserInvoice(Integer userId);
    /**
     * 新增用户发票信息
     */
    void insertUserInvoice(InvoiceDto invoiceDto);
    /**
     * 更新用户发票信息
     */
    void updateUserInvoice(InvoiceDto invoiceDto);
    /**
     * 获取用户发票信息
     */
    InvoiceVo getUserInvoice(Integer userId);
    /**
     * 根据商品Id获取下单商品信息
     * @param productId 商品Id
     */
    CartItemVo getCartItemByProductId(Integer productId);
}
