package com.caimei365.order.mapper;

import com.caimei365.order.model.dto.SellerCartDto;
import com.caimei365.order.model.po.SellerCartPo;
import com.caimei365.order.model.vo.CartItemVo;
import com.caimei365.order.model.vo.CartShopVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/29
 */
@Mapper
public interface CartSellerMapper {
    /**
     * 协销购物车供应商列表
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     */
    List<CartShopVo> getSellerCartShops(Integer serviceProviderId, Integer clubId);
    /**
     * 协销购物车供应商下商品列表
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     * @param shopId             供应商Id
     */
    List<CartItemVo> getSellerCartProducts(Integer serviceProviderId, Integer clubId, Integer shopId);

    /**
     * 获取协销购物车商品种类
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     */
    Integer getSellerCartCount(Integer serviceProviderId, Integer clubId);
    /**
     * 获取协销购物车
     * @param sellerCartDto clubId,productId,serviceProviderId
     */
    SellerCartPo getSellerCart(SellerCartDto sellerCartDto);
    /**
     * 根据Id获取协销购物车
     * @param id 购物车id
     */
    SellerCartPo getSellerCartById(Integer id);
    /**
     * 更新购物车
     * @param sellerCartPo SellerCartPo
     */
    void updateSellerCart(SellerCartPo sellerCartPo);
    /**
     * 添加购物车
     * @param sellerCartPo SellerCartPo
     */
    void insertSellerCart(SellerCartPo sellerCartPo);
    /**
     * 删除购物车
     * @param serviceProviderId  协销Id
     * @param cartIds 购物车Id集合
     */
    void deleteSellerCartByIds(@Param("serviceProviderId") Integer serviceProviderId,@Param("cartIds") List<Integer> cartIds);
    /**
     * 删除购物车
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     * @param productIds         商品Id集合
     */
    void deleteSellerCartByProductIds(@Param("serviceProviderId") Integer serviceProviderId,@Param("clubId") Integer clubId, @Param("productIds") List<String> productIds);
    /**
     * 根据商品Ids获取购物车供应商列表
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     * @param productIds      商品Ids
     */
    List<CartShopVo> getCartShopsByProductIds(@Param("serviceProviderId") Integer serviceProviderId, @Param("clubId") Integer clubId, @Param("productIds") List<String> productIds);
    /**
     * 根据商品Ids获取购物车已上架商品列表
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     * @param shopId             供应商Id
     * @param productIds        商品Ids
     */
    List<CartItemVo> getCartProductsByProductIds(@Param("serviceProviderId") Integer serviceProviderId, @Param("clubId") Integer clubId, @Param("shopId") Integer shopId, @Param("productIds") List<String> productIds);
    /**
     * 根据商品Id获取二手商品
     * @param productId      商品Id
     */
    CartItemVo getSecondByProductId(Integer productId);
    /**
     * 获取订单里用户Id
     * @param orderId 订单Id
     */
    Integer getOrderUserIdByOrderId(Integer orderId);
    /**
     * 获取订单里用户Id
     * @param orderId 订单Id
     */
    Integer getOrderClubIdByOrderId(Integer orderId);
    /**
     * 获取订单商品的商品数量列表
     * @param orderId 订单Id
     */
    List<CartItemVo> getOrderProductNum(Integer orderId);
}
