package com.caimei365.order.mapper;

import com.caimei365.order.model.po.BalanceRecordPo;
import com.caimei365.order.model.po.CouponOrderRecordPo;
import com.caimei365.order.model.po.OrderPo;
import com.caimei365.order.model.po.OrderProductPo;
import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/19
 */
@Mapper
public interface OrderClubMapper {
    /**
     * 判断订单是否存在
     * @param orderId 订单Id
     */
    int countMainOrderByOrderId(Integer orderId);
    /**
     * 根据订单Id查询订单
     * @param orderId 订单Id
     */
    OrderPo getMainOrderByOrderId(Integer orderId);
    /**
     * 根据订单Id查询订单商品列表
     * @param orderId 订单Id
     */
    List<OrderProductPo> getOrderProductList(Integer orderId);
    /**
     * 设置二手仪器已售
     * @param productId 商品Id
     */
    void updateSecondStatus(Integer productId, Integer sold);
    /**
     * 根据用户Id更新用户余额
     * @param userId 用户Id
     */
    void updateUserMoneyByUserId(double userMoney, Integer userId);
    /**
     * 根据用户Id更新用户可用余额
     * @param userId 用户Id
     */
    void updateAbleUserMoneyByUserId(double ableUserMoney, Integer userId);
    /**
     * 更新订单状态
     * @param order 主订单
     */
    void updateOrderStatus(OrderPo order);
    /**
     * 取消 余额收支记录
     */
    void cancelBalanceRecord(BalanceRecordPo balanceRecord);
    /**
     * 取消 订单
     */
    void cancelMainOrder(OrderPo order);
    /**
     * 逻辑删除主订单
     */
    void deleteMainOrder(Integer orderId, String note);
    /**
     * 逻辑删除子订单
     * @param shopOrderIds 子订单Ids
     */
    void deleteShopOrder(@Param("shopOrderIds") List<String> shopOrderIds);
    /**
     * 获取收款记录Id列表
     * @param orderId 订单Id
     */
    List<String> getOrderReceiptIds(Integer orderId);
    /**
     * 逻辑删除订单与收款记录关系表
     * @param orderId 订单Id
     */
    void deleteOrderReceiptRelation(Integer orderId);
    /**
     * 逻辑删除收款记录表
     * @param receiptIds 收款记录Ids
     */
    void deleteDiscernReceipt(@Param("receiptIds") List<String> receiptIds);
    /**
     * 获取付款记录Id列表
     * @param shopOrderIds 子订单Ids
     */
    List<String> getPayShopIds(@Param("shopOrderIds") List<String> shopOrderIds);
    /**
     * 逻辑删除订单与付款记录关系表
     * @param shopOrderIds 子订单Ids
     */
    void deletePayShopRecord(@Param("shopOrderIds") List<String> shopOrderIds);
    /**
     * 逻辑删除付款记录表
     * @param payIds 付款记录Ids
     */
    void deletePayShop(@Param("payIds") List<String> payIds);
    /**
     * 更新物流状态
     * @param orderId 订单Id
     */
    void updateLogisticsBatch(Integer orderId);
    /**
     * 机构订单列表
     * @param userId     用户Id
     * @param orderState 1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderNo    订单编号
     * @param beginTime  开始时间
     * @param endTime    截止时间
     */
    List<OrderVo> getOrderList(Integer userId, Integer orderState, String orderNo, String beginTime, String endTime);
    /**
     * 根据关键词搜索订单
     * @param searchWord 关键词
     * @param userId     用户Id
     */
    List<OrderVo> getOrderListByKeyword(String searchWord, Integer userId);
    /**
     * 根据用户Id查找订单搜索历史记录
     * @param userId 用户Id
     */
    List<SearchHistoryVo> getOrderSearchHistory(Integer userId);
    /**
     * 根据用户Id删除订单搜索历史记录
     * @param userId 用户Id
     */
    void deleteOrderSearchHistory(Integer userId);
    /**
     * 退款记录
     * @param orderId 订单Id
     */
    List<ReturnedPurchaseVo> getReturnedPurchaseList(Integer orderId);
    /**
     * 售后条款
     * @param clauseId 条款Id
     */
    ClauseVo getClauseById(Integer clauseId);
    /**
     * 获取物流批次子订单Id列表
     * @param orderId 订单Id
     * @param id 物流批次Id
     */
    List<String> getBatchShopOrderIds(Integer orderId, Integer id);
    /**
     * 获取数据库已有分享码
     * @param orderId 订单Id
     */
    OrderShareCodeVo getOrderCodeById(Integer orderId);
    /**
     * 删除分享码使用记录
     * @param orderId 订单Id
     */
    void deleteOrderShareCodeRecord(Integer orderId);
    /**
     * 更新数据库的分享码
     */
    void updateOrderShareCode(OrderShareCodeVo orderShareCode);
    /**
     * 新增数据库的分享码
     */
    void insertOrderShareCode(OrderShareCodeVo orderShareCode);
    /**
     * 根据openId统计数据库内是否有有效分享码
     */
    int countShareCodeRecord(Integer orderId, String openId);
    /**
     * 保存分享码使用记录
     */
    void insertOrderShareCodeRecord(OrderShareCodeRecordVo codeRecord);
    /**
     * 根据clubCouponId获取优惠券
     */
    CouponVo getClubCouponById(Integer clubCouponId);
    /**
     * 修改优惠券使用情况
     */
    void updateCouponClubStatus(Integer clubCouponId, Integer orderId);
    /**
     * 查询订单优惠信息
     */
    CouponOrderRecordPo getOrderCouponRecord(Integer orderId);
}
