package com.caimei365.order.mapper;

import com.caimei365.order.model.vo.OrderVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/19
 */
@Mapper
public interface OrderSellerMapper {
    /**
     * 获取该协销下的机构用户Id
     * @param serviceProviderId 协销Id
     */
    List<Integer> getClubUserIds(Integer serviceProviderId);

    /**
     * 订单列表
     * @param userIds 用户Ids
     * @param listType          1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderSubmitType   1：自主订单， 2:协销或后台订单
     * @param orderNo           订单编号
     */
    List<OrderVo> getOrderList(@Param("userIds") List<Integer> userIds, Integer listType, @Param("orderSubmitType") Integer orderSubmitType, @Param("orderNo") String orderNo);

    /**
     * 根据机构Id获取协销Id
     * @param clubId 机构Id
     */
    Integer getSpIdByClubId(Integer clubId);

    /**
     * 机构总订单列表
     * @param clubId            机构Id
     * @param listType          1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderSubmitType   1：自主订单， 2:协销或后台订单
     */
    List<OrderVo> getOrderListClub(Integer clubId, Integer listType, Integer orderSubmitType);

    /**
     * 更新协销查看机构订单时间
     * @param clubId 机构Id
     * @param date   时间
     */
    void updateClubLastCheckOrderDate(Integer clubId, Date date);

    /**
     * 获取协销用户Id
     * @param serviceProviderId 协销Id
     */
    Integer getUserIdBySpId(Integer serviceProviderId);

    /**
     * 协销根据关键词搜索订单
     * @param searchWord 关键词
     * @param userIds     用户Id
     */
    List<OrderVo> getOrderListByKeyword(@Param("userIds") List<Integer> userIds, @Param("searchWord") String searchWord);
}
