package com.caimei365.order.mapper;

import com.caimei365.order.model.po.OrderReceiptRelationPo;
import com.caimei365.order.model.po.PayShopPo;
import com.caimei365.order.model.po.PayShopRecordPo;
import com.caimei365.order.model.po.SplitAccountPo;
import com.caimei365.order.model.vo.OrderPayLinkVo;
import com.caimei365.order.model.vo.OrderVo;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/29
 */
@Mapper
public interface PayOrderMapper {
    /**
     * 获取线上支付全局开关状态
     */
    Integer getPayOnLineSwitch();
    /**
     * 查询审核中的退款退货数量
     * @param orderId 订单Id
     */
    int getRefundCount(Integer orderId);
    /**
     * 更新订单状态
     * @param order 主订单
     */
    void updateOrderStatus(OrderVo order);
    /**
     * 根据用户Id更新余额
     * @param userMoney      用户余额
     * @param ableUserMoney 用户可用余额
     * @param userId 用户Id
     */
    void updateMoneyByUserId(Double userMoney, Double ableUserMoney, Integer userId);
    /**
     * 支付链接
     * @param orderId 订单Id
     * @param amount  本次待付款金额
     */
    OrderPayLinkVo getOrderPayLink(Integer orderId, Double amount);
    /**
     * 支付链接
     * @param linkLogo  链接标识
     */
    OrderPayLinkVo getPayLinkByLinkLogo(String linkLogo);
    /**
     * 新增支付链接
     */
    void insertOrderPayLink(OrderPayLinkVo orderPayLink);
    /**
     * 更新支付链接
     */
    void updateOrderPayLink(OrderPayLinkVo orderPayLink);
    /**
     * 修改支付链接状态
     */
    void updateOrderPayLinkStatus(OrderPayLinkVo orderPayLink);
    /**
     * 订单促销Id
     * @param orderId 订单Id
     */
    Integer getOrderPromotionsId(Integer orderId);
    /**
     * 付款次数
     * @param orderId 订单Id
     */
    Integer getDbPaySuccessCounter(Integer orderId);
    /**
     * 保存分账详情
     */
    void insertSplitAccount(SplitAccountPo splitAccount);
    /**
     * 更新二手详情
     */
    void updateSeconHandDetail(Integer productId, String payType, double payAmount, String payFormData);
    /**
     * 修改分账付款状态
     */
    void updateSplitAccountByPay(String mbOrderId);
    /**
     * 修改收款分账状态
     */
    void updateBySplitStatus(String mbOrderId);
    /**
     * 查询分账给子商户号
     */
    List<SplitAccountPo> getSplitAccountList(String mbOrderId);
    /**
     * 查询已付供应商金额
     */
    Double getPaidShopAmount(Integer shopOrderId);
    /**
     * 修改子订单付款状态及付款金额
     */
    void updateShopOrderByPayStatus(Integer shopOrderId, Double paidShop, Integer payStatus);
    /**
     * 修改主订单付款状态
     */
    void updateOrderByPayStatus(Integer orderId, Integer payStatus);
    /**
     * 保存付款单表
     */
    void insertPayShop(PayShopPo payShop);
    /**
     * 保存付供应商记录
     */
    void insertPayShopRecord(PayShopRecordPo shopRecord);
    /**
     * 查询未分账已支付收款
     */
    List<OrderReceiptRelationPo> getUndividedPaidReceipt(String currentTime);
    /**
     * 查询已支付金额
     * @param orderProductId 订单商品Id
     */
    Double getOrderProductPaidAmount(Integer orderProductId);
    /**
     * 查询子商户商编
     * @param shopId 供应商Id
     */
    String getShopCommercialCode(Integer shopId);
    /**
     * 查询已支付运费
     */
    Double getPaidShipping(Integer orderId, Integer shopId);
    /**
     * 获取会员套餐价格
     * @param id 会员套餐Id
     */
    Double getVipPackagePriceById(Integer id);
}
