package com.caimei365.order.mapper;

import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/8/5
 */
@Mapper
public interface ShipMapper {
    /**
     * 机构订单列表
     */
    List<ShopOrderVo> getShopOrderList(Integer shopId, Integer sendOutStatus, Integer payStatus, String shopOrderNo, String receiver);
    /**
     * 供应商结算列表
     */
    List<SettlementVo> getSettlementList(Integer shopId, Integer payStatus, String shopOrderNo, String name);
    /**
     * 发货页面子订单数据
     * @param shopOrderId 子订单Id
     */
    ShopOrderVo getShopOrder(Integer shopOrderId);
    /**
     * （收款买家）收款状态：1待收款、2部分收款、3已收款
     * @param orderId 订单Id
     */
    Integer getOrderReceiptStatus(Integer orderId);
    /**
     * 物流公司
     * @param value 快递公司代码
     */
    List<CompanyVo> getLogisticsCompany(String value);
    /**
     * 统计物流批次
     */
    int countLogisticsBatch(Integer orderId);
    /**
     * 保存 物流批次
     * @param logisticsBatch LogisticsBatchPo
     */
    void insertLogisticsBatch(LogisticsBatchPo logisticsBatch);
    /**
     * 更新物流批次
     */
    void updateLogisticsBatchRemark(LogisticsBatchVo logisticsBatch);
    /**
     * 订单商品
     * @param orderProductId 订单商品Id
     */
    OrderProductVo getOrderProductById(Integer orderProductId);
    /**
     * 保存 发货物流记录
     * @param logisticsRecord LogisticsRecordPo
     */
    void insertLogisticsRecord(LogisticsRecordPo logisticsRecord);
    /**
     * 保存订单商品未出库数量
     * @param orderProductId 订单商品Id
     * @param notOutStore 未出库数量
     */
    void updateNotOutStore(Integer orderProductId, int notOutStore);
    /**
     * 保存 物流跟踪信息
     * @param logisticsInformation LogisticsInformationPo
     */
    void insertLogisticsInformation(LogisticsInformationPo logisticsInformation);
    /**
     * 修改子订单发货状态
     * @param shopOrderId 子订单Id
     * @param outStoreNum 已经发货的商品数量
     * @param outStoreTimes 子订单发货批次 默认值 = 0 即没过发货
     * @param sendOutStatus 发货状态：1待发货、2部分发货、3已发货
     */
    void updateShopOrderShip(Integer shopOrderId, Integer outStoreNum, Integer outStoreTimes, Integer sendOutStatus);
    /**
     * 子订单发货状态列表
     * @param orderId 订单Id
     */
    List<Integer> getAllSendOutStatus(Integer orderId);
    /**
     * 修改主订单发货状态
     * @param orderId 订单Id
     * @param sendOutStatus 发货状态：1待发货、2部分发货、3已发货
     * @param status 订单状态
     */
    void updateOrderShip(Integer orderId, Integer sendOutStatus, Integer status);
    /**
     * 获取订单Id
     * @param shopOrderId 子订单Id
     */
    Integer getOrderIdByShopOrder(Integer shopOrderId);
    /**
     * 发货物流批次
     * @param logisticsBatchId 发货物流批次Id
     */
    LogisticsBatchVo getLogisticsBatchById(Integer logisticsBatchId);
    /**
     * 删除物流信息
     * @param logisticsInfoId 物流信息Id
     */
    void deleteLogisticsInfoByInfoId(Integer logisticsInfoId);
    /**
     * 删除物流批次
     * @param logisticsBatchId 发货物流批次Id
     */
    void deleteLogisticsBatchByBatchId(Integer logisticsBatchId);
    /**
     * 删除物流记录
     * @param logisticsBatchId 发货物流批次Id
     */
    void deleteLogisticsRecordByBatchId(Integer logisticsBatchId);
    /**
     * 删除物流信息
     * @param logisticsBatchId 发货物流批次Id
     */
    void deleteLogisticsInfoByBatchId(Integer logisticsBatchId);
    /**
     * 当前批次发货商品数据
     * @param logisticsBatchId 发货物流批次Id
     */
    List<LogisticsRecordVo> getLogisticsRecord(Integer logisticsBatchId);
    /**
     * 结算记录
     * @param shopOrderId 子订单Id
     */
    List<PayShopRecordPo> getPayShopRecordList(Integer shopOrderId);
    /**
     * 获取客服名称
     */
    String getConsignmentName(String openId);
    /**
     * 子订单所属供应商Id
     */
    Integer getShopIdByShopOrderId(Integer shopOrderId);
    /**
     * 采美供应商发货分享码使用记录Id
     */
    Integer getShipShareCodeRecordId(String openId, Integer shopOrderId);
    /**
     * 采美供应商发货分享码
     */
    OrderShareCodeVo getShipShareCode(String shareCode, Integer shopOrderId);
    /**
     * 保存发货分享码使用记录
     */
    void insertShipShareCodeRecord(OrderShareCodeRecordVo codeRecord);
    /**
     * 删除发货分享码使用记录
     */
    void deleteShipShareCodeRecord(Integer shopOrderId);
    /**
     * 新增发货分享码
     */
    void insertShipShareCode(OrderShareCodeVo shareCode);
    /**
     * 更新发货分享码
     */
    void updateShipShareCode(OrderShareCodeVo orderCodeVo);
    /**
     * 查看商品资质信息
     */
    List<QualificationVo> getProductQualification(Integer logisticsBatchId);
    /**
     * 查询商品资质图片
     */
    List<String> getQualificationImage(Integer recordId);
    /**
     * 查询商品资质文件
     */
    List<QualificationPo> getQualificationFile(Integer recordId);
    /**
     * 删除商品资质文件
     * @param recordId 商品发货记录id
     */
    void deleteQualificationFile(Integer recordId);
    /**
     * 删除资质图片
     * @param recordId 商品发货记录id
     */
    void deleteQualificationImage(Integer recordId);
    /**
     * 保存资质文件
     */
    void insertQualificationFile(QualificationPo qualificationFile);
    /**
     * 保存资质图片
     */
    void insertQualificationImage(String image, Integer recordId);
    /**
     * 更新发货物流记录SN码
     */
    void updateLogisticsRecordSn(Integer recordId, String sn);
}
