package com.caimei365.order.mapper;

import com.caimei365.order.model.bo.OrderParamBo;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.AddressVo;
import com.caimei365.order.model.vo.LadderPriceVo;
import com.caimei365.order.model.vo.PromotionsVo;
import org.apache.ibatis.annotations.Mapper;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/6
 */
@Mapper
public interface SubmitMapper {
    /**
     * 获取运营人员Id
     * @param unionId    运营人员unionId
     * @param userId     机构用户Id
     */
    Integer getOperationIdByUnionId(String unionId, Integer userId);
    /**
     * 根据用户Id获取用户余额信息
     * @param userId 用户Id
     */
    OrderParamBo getOrderUserBoById(Integer userId);
    /**
     * 获取协销用户Id
     * @param serviceProviderId 协销Id
     */
    Integer getServiceProviderUserId(Integer serviceProviderId);
    /**
     * 获取数据库商品信息
     * @param productId 商品Id
     */
    OrderProductPo getProductDetails(Integer productId);
    /**
     * 抵扣后更新用户余额
     * @param userMoney 账户余额
     * @param ableUserMoney 账户实际可用余额
     * @param userId 机构用户Id
     */
    void updateUserMoney(Double userMoney, Double ableUserMoney, Integer userId);
    /**
     * 获取条款名称
     * @param clauseId 条款Id
     */
    String getClauseNameById(Integer clauseId);
    /**
     * 保存主订单数据
     */
    void insertMainOrder(OrderPo order);
    /**
     * 保存子订单数据
     */
    void insertShopOrder(OrderShopPo shopOrder);
    /**
     * 保存 订单商品
     */
    void insertOrderProduct(OrderProductPo orderProduct);
    /**
     * 保存订单促销
     */
    void insertOrderPromotions(PromotionsVo promotions);
    /**
     * 保存 订单阶梯价格
     */
    void insertLadderPrices(LadderPriceVo ladderPriceVo);
    /**
     * 更新订单的子订单Ids
     * @param shopOrderIds 子订单Ids
     * @param orderId 订单Id
     */
    void updateShopOrderIds(String shopOrderIds, Integer orderId);
    /**
     * 查询订单增值税发票
     */
    InvoicePo getOrderInvoice(Integer orderId);
    /**
     * 更新订单增值税发票
     */
    void updateOrderInvoice(InvoicePo orderInvoice);
    /**
     * 保存 订单发票信息
     */
    void insertOrderInvoice(InvoicePo invoice);
    /**
     * 获取用户地信息
     * @param addressId 地址Id
     */
    AddressVo getAddressDetailById(Integer addressId);
    /**
     * 保存 订单用户地址
     */
    void insertOrderUserInfo(OrderUserInfoPo userInfo);
    /**
     * 保存订单优惠记录
     */
    void insertCouponOrderRecord(CouponOrderRecordPo orderRecord);
    /**
     * 获取超级会员优惠商品详情
     * @param productId 商品id
     * @return
     */
    SvipProductPo getSvipProductDetails(Integer productId);
}
