package com.caimei365.order.service;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.AddressDto;
import com.caimei365.order.model.vo.AddressSelectVo;
import com.caimei365.order.model.vo.AddressVo;
import com.github.pagehelper.PageInfo;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/2
 */
public interface AddressService {

    /**
     * 收货地址列表
     * @param userId 用户Id
     */
    ResponseJson<PageInfo<AddressVo>> getUserAddressList(Integer userId, int pageNum, int pageSize);

    /**
     * 收货地址下拉选项列表
     * @param type     选项类型：0省（默认），1市，2区
     * @param parentId 父级地址Id
     */
    ResponseJson<List<AddressSelectVo>> getSelectAddress(Integer type, Integer parentId);

    /**
     * 所有地址下拉完整数据
     */
    ResponseJson<List<AddressSelectVo>> getAllSelectAddress();

    /**
     * 添加收货地址
     *
     * @param addressDto {
     *                   userId       用户ID
     *                   addressId    用户地址Id
     *                   townId       区ID
     *                   address      详细地址
     *                   name         收货人
     *                   mobile       手机
     *                   defaultFlag  是否默认收货地址(0 不是默认,1 默认)
     */
    ResponseJson<Integer> addUserAddress(AddressDto addressDto);

    /**
     * 删除收货地址
     *
     * @param addressDto {
     *                   userId       用户ID
     *                   addressId    用户地址Id
     */
    ResponseJson<Integer> deleteUserAddress(AddressDto addressDto);

    /**
     * 设为默认地址
     * @param userId    用户ID
     * @param addressId 用户地址Id
     */
    ResponseJson<Integer> setDefaultAddress(Integer userId, Integer addressId);
}
