package com.caimei365.order.service;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.CartDto;
import com.caimei365.order.model.dto.InvoiceDto;
import com.caimei365.order.model.vo.InvoiceVo;

import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/25
 */
public interface CartClubService {

    /**
     * 购物车列表详细数据
     * @param userId 用户Id
     * @param source 来源 : 1 网站 ; 2 小程序
     */
    ResponseJson<Map<String, Object>> getShoppingCartList(Integer userId, Integer source);

    /**
     * 网站顶部购物车数据
     * @param userId 用户Id
     */
    ResponseJson<Map<String, Object>> getShoppingCartHead(Integer userId);

    /**
     * 添加购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     * }
     */
    ResponseJson<Integer> addShoppingCart(CartDto cartDto);

    /**
     * 更新购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     * }
     */
    ResponseJson<Integer> updateShoppingCart(CartDto cartDto);

    /**
     * 删除购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productIds   商品ids,逗号隔开
     * }
     */
    ResponseJson<Integer> deleteShoppingCart(CartDto cartDto);

    /**
     * 购物车结算
     *
     * @param  userId       用户ID
     * @param  productIds   商品ids,逗号隔开
     * @param source 来源 : 1 网站 ; 2 小程序
     */
    ResponseJson<Map<String, Object>> settlementShoppingCart(Integer userId, String productIds, Integer source);

    /**
     * 立即购买
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     *                source       来源 : 1 网站 ; 2 小程序
     * }
     */
    ResponseJson<Map<String, Object>> buyNowProduct(CartDto cartDto);

    /**
     * 获取结算商品运费
     * @param  userId       用户ID
     * @param  productIds   商品ids,逗号隔开
     * @param townId        地区Id
     */
    ResponseJson<Map<String, Object>> getProductsPostage(Integer userId, String productIds, Integer townId);

    /**
     * 获取用户发票信息
     * @param userId 用户Id
     */
    ResponseJson<InvoiceVo> getUserInvoice(Integer userId);

    /**
     * 更新用户发票
     *
     * @param invoiceDto {
     *                   userId:用户Id
     *                   id:id
     *                   invoiceTitle:单位名，发票抬头
     *                   corporationTaxNum: 企业税号、纳税人识别号
     *                   registeredAddress:注册地址
     *                   registeredPhone:注册电话
     *                   bankAccountNo:开户银行账户
     *                   openBank:开户银行
     *                   }
     */
    ResponseJson<Integer> updateUserInvoice(InvoiceDto invoiceDto);

    /**
     * 供应商优惠券
     *
     * @param userId 用户id
     * @param shopId 供应商id
     * @param source 来源 : 1 网站 ; 2 小程序
     * @param status 状态: 1未领取 2已领取
     */
    ResponseJson<Map<String, Object>> getShopCoupons(Integer userId, Integer shopId, Integer source, Integer status);
}
