package com.caimei365.order.service;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.AgainBuyDto;
import com.caimei365.order.model.dto.SellerCartDto;

import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/29
 */
public interface CartSellerService {

    /**
     * 协销购物车列表数据
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     * @param againBuyProductIds 再来一单商品Ids
     * @param pageNum            页码
     * @param pageSize           每页数量
     */
    ResponseJson<Map<String, Object>> getSellerCartList(Integer serviceProviderId, Integer clubId, String againBuyProductIds, int pageNum, int pageSize);

    /**
     * 获取协销购物车商品种类
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     */
    ResponseJson<Integer> getSellerCartCount(Integer serviceProviderId, Integer clubId);

    /**
     * 协销添加购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      clubId       机构ID
     *                      type         商品类型：1单一商品，2组合商品
     *                      productId    单一商品id
     *                      productNum   单一商品数量
     *                      productInfo  组合商品信息：Json字符串格式[{"id":4351,"num":1},{}]
     * }
     */
    ResponseJson<Integer> addShoppingCart(SellerCartDto sellerCartDto);

    /**
     * 协销更新购物车
     *
     * @param sellerCartDto {
     *                      id                购物车Id
     *                      serviceProviderId 协销Id
     *                      productNum        商品数量
     *                      }
     */
    ResponseJson<Integer> updateSellerCart(SellerCartDto sellerCartDto);

    /**
     * 协销删除购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      cartIds           购物车IDs
     * }
     */
    ResponseJson<Integer> deleteSellerCart(SellerCartDto sellerCartDto);

    /**
     * 协销购物车结算
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     * @param productIds        商品Ids
     */
    ResponseJson<Map<String, Object>> settlementSellerCart(Integer serviceProviderId, Integer clubId, String productIds);

    /**
     * 协销二手商品立即购买
     * @param serviceProviderId 协销Id
     * @param clubId       机构Id
     * @param productId    商品Id
     * @param productCount 商品数量
     */
    ResponseJson<Map<String, Object>> settlementSecond(Integer serviceProviderId, Integer clubId, Integer productId, Integer productCount);

    /**
     * 协销再来一单
     *
     * @param againBuyDto {
     *                    serviceProviderId      协销Id
     *                    orderId                订单Id
     *                    confirmFlag            确认标识 0未确认 1已确认-将失效商品以外的正常商品加入购物车
     *                    }
     */
    ResponseJson<Map<String, Object>> addCartBuyAgain(AgainBuyDto againBuyDto);
}
