package com.caimei365.order.service;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.vo.*;
import com.github.pagehelper.PageInfo;
import org.springframework.http.HttpHeaders;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/19
 */
public interface OrderClubService {
    /**
     * 机构订单列表
     * 
     * @param userId     用户Id
     * @param orderState 1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderNo    订单编号
     * @param beginTime  开始时间
     * @param endTime    截止时间
     * @param pageNum    页码
     * @param pageSize   每页数量
     */
    ResponseJson<PageInfo<OrderVo>> getOrderList(Integer userId, Integer orderState, String orderNo, String beginTime, String endTime, int pageNum, int pageSize);
    /**
     * 根据关键词搜索订单
     * @param userId     用户Id
     * @param searchWord 搜索关键词
     * @param pageNum    页码
     * @param pageSize   每页数量
     */
    ResponseJson<PageInfo<OrderVo>> getOrderListByKeyword(Integer userId, String searchWord, int pageNum, int pageSize);
    /**
     * 机构订单详情
     * @param userId     用户Id
     * @param orderId    订单Id
     */
    ResponseJson<Map<String, Object>> getOrderDetail(Integer userId, Integer orderId);
    /**
     * 机构确认订单
     * @param orderId  订单Id
     */
    ResponseJson<Void> confirmMainOrder(Integer orderId);
    /**
     * 机构取消订单
     * @param orderId  订单Id
     * @param remark    备注（订单取消原因）
     */
    ResponseJson<Void> cancelMainOrder(Integer orderId, String remark);
    /**
     * 机构删除订单
     * @param orderId  订单Id
     */
    ResponseJson<Void> deleteMainOrder(Integer orderId);
    /**
     * 机构确认收货
     * @param orderId 订单Id
     */
    ResponseJson<Void> receiveGoods(Integer orderId);
    /**
     * 根据用户Id查找订单搜索历史记录
     * @param userId  用户Id
     */
    ResponseJson<List<SearchHistoryVo>> getOrderSearchHistory(Integer userId);
    /**
     * 根据用户Id删除订单搜索历史记录
     * @param userId  用户Id
     */
    ResponseJson<Void> deleteOrderSearchHistory(Integer userId);
    /**
     * 物流详情
     * @param orderId 订单Id
     */
    ResponseJson<List<LogisticsBatchVo>> getOrderLogistics(Integer orderId);
    /**
     * 分享订单，获取分享码
     * @param orderId 订单Id
     */
    ResponseJson<String> getOrderShareCode(Integer orderId);
    /**
     * 分享订单，用户身份
     * @param orderId 订单Id
     * @param userId 用户Id
     * @param serviceProviderId 协销Id
     * @param code 微信code
     * @param encryptedData 微信加密数据
     * @param iv 微信加密算法的初始向量
     * @param headers HttpHeaders
     */
    ResponseJson<String> identityOrderShareCode(Integer orderId, Integer userId, Integer serviceProviderId, String code, String encryptedData, String iv, HttpHeaders headers);
    /**
     * 分享订单，校验分享码
     * @param orderId 订单Id
     * @param shareCode 分享码
     * @param code 微信code
     * @param encryptedData 微信加密数据
     * @param iv 微信加密算法的初始向量
     * @param headers HttpHeaders
     */
    ResponseJson<String> checkOrderShareCode(Integer orderId, String shareCode, String code, String encryptedData, String iv, HttpHeaders headers);
    /**
     * 分享订单，初始商品数据
     * @param orderId 订单Id
     */
    ResponseJson<List<OrderProductVo>> getOrderShareInitProduct(Integer orderId);
    /**
     * 二手订单 确认打款供应商
     * @param orderId 订单Id
     * @param userId 用户Id
     */
    ResponseJson<Void> secondPaymentConfirm(Integer orderId, Integer userId);
    /**
     * 根据售后条款ID查看售后条款内容
     * @param clauseId 售后条款Id
     */
    ResponseJson<ClauseVo> getClauseById(Integer clauseId);
}
