package com.caimei365.order.service;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.PayDto;
import com.caimei365.order.model.dto.PayLinkDto;
import org.springframework.http.HttpHeaders;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/29
 */
public interface PayOrderService {
    /**
     * 订单支付前效验付款规则
     * @param orderId 订单Id
     */
    ResponseJson<Map<String, Object>> payBeforeCheck(Integer orderId);
    /**
     * 订单支付,余额抵扣
     * @param orderId 订单Id
     */
    ResponseJson<Map<String, Object>> balanceDeduction(Integer orderId);
    /**
     * 获取线上支付全局开关状态
     */
    ResponseJson<Integer> getPayOnLineSwitch();
    /**
     * 收银台数据显示
     * @param orderId 订单Id
     */
    ResponseJson<Map<String, Object>> getCheckoutCounter(Integer orderId);
    /**
     * 生成网银支付链接
     * @param payLinkDto {
     *          orderId       订单ID
     * }
     */
    ResponseJson<String> getPayLink(PayLinkDto payLinkDto);
    /**
     * 支付链接重定向到页面
     * @param linkLogo 链接标识
     */
    void jumpPayPage(String linkLogo, HttpServletResponse response) throws IOException;
    /**
     * 重定向页面的支付数据
     * @param linkLogo 链接标识
     */
    ResponseJson<Map<String, Object>> linkPayData(String linkLogo);
    /**
     * 微信线上支付
     * @param payDto {
     *               orderId      订单ID
     *               payAmount     支付金额，单位分，必须大于2
     *               code          微信小程序code
     *               state         微信公众号state参数
     *               returnUrl     页面回调地址
     * }
     * @param headers HttpHeaders
     */
    ResponseJson<JSONObject> payByWeChat(PayDto payDto, HttpHeaders headers);
    /**
     * 支付宝线上支付
     * @param payDto {
     *               orderId      订单ID
     *               payAmount     支付金额，单位分，必须大于2
     *               returnUrl     页面回调地址
     * }
     */
    ResponseJson<JSONObject> payByAlipay(PayDto payDto, HttpHeaders headers);
    /**
     * 银联线上支付
     * @param payDto {
     *               orderId       订单ID
     *               payAmount     支付金额，单位分，必须大于2
     *               returnUrl     页面回调地址
     *               bankCode      银行编码(银联支付使用)
     *               userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     * }
     */
    ResponseJson<JSONObject> payByUnionPay(PayDto payDto, HttpHeaders headers);
    /**
     * 支付异步通知回调
     */
    String paymentCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException;
    /**
     * 判断此次支付是否完成
     * @param orderId           订单id
     * @param paySuccessCounter 付款次数
     */
    ResponseJson<String> payWhetherSuccess(Integer orderId, Integer paySuccessCounter);
    /**
     * 查询本次支付订单结果
     * @param mbOrderId 平台唯一流水号
     */
    ResponseJson<JSONObject> getPayOrderResult(String mbOrderId);
    /**
     * 延时分账
     */
    void delayedSplitting(String callUrl);
    /**
     * 延时分账异步通知回调
     */
    String delayedSplittingCallback(String data);
//    /**
//     * 手动临时分账
//     */
//    String temporarySplitting();
//    /**
//     * 手动临时分账异步通知回调
//     */
//    String tempSplittingCallback(String data);

}
