package com.caimei365.order.service.impl;

import com.caimei365.order.components.ProductService;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.CartClubMapper;
import com.caimei365.order.mapper.OrderCommonMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.CartDto;
import com.caimei365.order.model.dto.InvoiceDto;
import com.caimei365.order.model.po.CartPo;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.CartClubService;
import com.caimei365.order.utils.MathUtil;
import com.google.common.collect.Lists;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/25
 */
@Slf4j
@Service
public class CartClubServiceImpl implements CartClubService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private CartClubMapper cartClubMapper;
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Resource
    private ProductService productService;

    /**
     * 购物车列表详细数据
     *
     * @param userId 用户Id
     * @param source 来源 : 1 网站 ; 2 小程序
     */
    @Override
    public ResponseJson<Map<String, Object>> getShoppingCartList(Integer userId, Integer source) {
        /*
         * 初始化返回数据
         */
        // 购物车供应商列表
        List<CartShopVo> shopInfoList = cartClubMapper.getCartShops(userId);
        // 失效商品列表
        List<CartItemVo> invalidList = new ArrayList<>();
        // 商品总数量
        AtomicInteger totalCount = new AtomicInteger(0);
        // 商品种类
        AtomicInteger kindCount = new AtomicInteger(0);
        // 统计商品总金额
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 统计总促销满减
        AtomicDouble reducedPrice = new AtomicDouble(0);
        // 统计总超级会员优惠
        AtomicDouble svipReducedPrice = new AtomicDouble(0);
        // 统计总划线价
        AtomicDouble totalOriginalPrice = new AtomicDouble(0);
        /*
         * 处理数据
         */
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        List<Integer> promotionsIds = new ArrayList<>();
        // 用户可用优惠券（总）
        List<CouponVo> couponList = orderCommonMapper.getClubCouponList(userId);
        List<CartItemVo> cartAllProducts = new ArrayList<>();
        // 用户身份
        Integer userIdentity = baseMapper.getIdentityByUserId(userId);
        // 超级会员标识
        Integer svipUserId = baseMapper.getSvipUserIdByUserId(userId);
        boolean svipUserFlag = null != svipUserId;
        if (null != shopInfoList && shopInfoList.size()>0) {
            // 删除空数据
            shopInfoList.removeIf(Objects::isNull);
            // 遍历供应商列表
            shopInfoList.forEach(shop -> {
                // 默认未选中状态（前端要求）
                shop.setIsChecked(false);
                // 该供应商下商品种类
                AtomicInteger shopKindCount = new AtomicInteger(0);
                // 该供应商总价
                AtomicDouble shopPrice = new AtomicDouble(0);
                // 该供应商下可参与店铺促销的商品总价
                AtomicDouble shopPromotionFee = new AtomicDouble(0);
                // 该供应商满减金额（供应商满减，单品满减）
                AtomicDouble shopReducedPrice = new AtomicDouble(0);
                // 该供应商svip优惠金额
                AtomicDouble shopSvipReducedPrice = new AtomicDouble(0);
                // 该供应商划线价
                AtomicDouble shopOriginalPrice = new AtomicDouble(0);
                // 供应商促销优惠活动
                PromotionsVo shopPromotion = baseMapper.getPromotionByShopId(shop.getShopId());
                // 供应商下商品列表
                List<CartItemVo> productList = cartClubMapper.getCartProductsByShopId(shop.getShopId(), userId);

                // 迭代器设置商品信息
                Iterator<CartItemVo> productIterator = productList.iterator();
                while (productIterator.hasNext()) {
                    CartItemVo cartItemVo = productIterator.next();
                    // 设置商品图片及税费
                    boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                    // 已上架商品
                    if (cartItemVo.getValidFlag() == 2) {
                        // 设置商品有效
                        cartItemVo.setStatus(0);
                        // 默认所有商品未选中状态（前端要求）
                        cartItemVo.setIsChecked(false);
                        // 价格是否可见
                        boolean priceVisible = (cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && userIdentity == 2) || svipUserFlag);
                        // 是否库存充足
                        boolean isStocked = (cartItemVo.getStock() != null && cartItemVo.getStock() > 0 && cartItemVo.getStock() >= cartItemVo.getMin() && cartItemVo.getStock() >= cartItemVo.getNumber());
                        if (priceVisible && isStocked) {
                            if (1 == cartItemVo.getSvipProductFlag()) {
                                // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                                // 超级会员设置商品优惠价
                                productService.setSvipProductPrice(cartItemVo, taxFlag, svipUserFlag);
                                if (svipUserFlag) {
                                    // 超级会员设置超级会员优惠总额 + 商品优惠金额
                                    shopSvipReducedPrice.set(MathUtil.add(shopSvipReducedPrice, cartItemVo.getSvipTotalReducedPrice()).doubleValue());
                                }
                            } else {
                                // 获取商品促销信息
                                PromotionsVo promotions = null;
                                // 没有店铺促销时，商品促销才有效
                                if (null == shopPromotion) {
                                    // 获取商品促销信息
                                    promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                                    /*
                                     * 设置商品促销优惠
                                     */
                                    if (null != promotions) {
                                        // 当前促销活动的价格计算列表
                                        List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                                        // 更新到总促销列表
                                        productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList);
                                        //单品满减-计算供应商总价/满减金额
                                        if (promotions.getType() == 1 && promotions.getMode() == 2) {
                                            BigDecimal totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice());
                                            if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) > -1) {
                                                // 如果满足促销条件，设置供应商价格-满减金额，满减总额 + 当前促销满减金额
                                                shopPrice.set(MathUtil.sub(shopPrice.get(), promotions.getReducedPrice()).doubleValue());
                                                shopReducedPrice.set(MathUtil.add(shopReducedPrice, promotions.getReducedPrice()).doubleValue());
                                            }
                                        }
                                        cartItemVo.setPromotions(promotions);
                                    }
                                }
                                if (null != promotions || null != shopPromotion) {
                                    // 商品处于活动状态
                                    cartItemVo.setActStatus(1);
                                    // 关闭阶梯价格，活动优先
                                    cartItemVo.setLadderFlag(0);
                                } else {
                                    if (cartItemVo.getLadderFlag() == 1) {
                                        // 设置阶梯价
                                        productService.setCartLadderPrices(cartItemVo, taxFlag);
                                    } else {
                                        // 复购价
                                        Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getProductId(), userId);
                                        if (null != repurchase && repurchase > 0) {
                                            if (taxFlag) {
                                                BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(cartItemVo.getPrice(), cartItemVo.getTaxRate()), 100, 2);
                                                cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                                            } else {
                                                cartItemVo.setPrice(repurchase);
                                            }
                                        }
                                    }
                                }
                            }
                            // 该供应商下价格累加
                            shopPrice.set(MathUtil.add(shopPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                            shopOriginalPrice.set(MathUtil.add(shopOriginalPrice.get(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getOriginalPrice())).doubleValue());
                            // 该供应商下可参与店铺促销的商品价格累加
                            if (!(1 == cartItemVo.getSvipProductFlag() && svipUserFlag)) {
                                // 超级会员购买svip商品不享受店铺促销优惠
                                shopPromotionFee.set(MathUtil.add(shopPromotionFee, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                            }
                            // 该供应商下商品种类 +1
                            shopKindCount.incrementAndGet();
                            // 购物车总数量 + 当前商品购买数量
                            totalCount.updateAndGet(v -> v + cartItemVo.getNumber());
                        } else {
                            // 失效商品
                            if (cartItemVo.getPriceFlag() == 1) {
                                // 未公开价格
                                cartItemVo.setStatus(6);
                            } else if (cartItemVo.getPriceFlag() == 2 && userIdentity == 4) {
                                // 价格仅会员可见
                                cartItemVo.setStatus(5);
                            } else if (cartItemVo.getStock() == null || cartItemVo.getStock() == 0) {
                                // 售罄
                                cartItemVo.setStatus(4);
                            } else if (cartItemVo.getStock() != null && (cartItemVo.getStock() < cartItemVo.getMin() || cartItemVo.getStock() < cartItemVo.getNumber())) {
                                // 库存不足
                                cartItemVo.setStatus(7);
                            }
                            invalidList.add(cartItemVo);
                            productIterator.remove();
                        }
                    } else {
                        // 失效商品
                        if (cartItemVo.getValidFlag() == 0) {
                            // 后台逻辑删除,已停售
                            cartItemVo.setStatus(1);
                            invalidList.add(cartItemVo);
                        } else if (cartItemVo.getValidFlag() == 10) {
                            // 已冻结，已丢失
                            cartItemVo.setStatus(2);
                            invalidList.add(cartItemVo);
                        } else if (cartItemVo.getValidFlag() == 3) {
                            // 已下架
                            cartItemVo.setStatus(3);
                            invalidList.add(cartItemVo);
                        }
                        //隐身商品validFlag = 9 不加入失效商品，直接去除
                        productIterator.remove();
                    }
                }
                cartAllProducts.addAll(productList);

                // 供应商下商品种类
                shop.setCount(shopKindCount.get());
                if (shopKindCount.get() > 0) {
                    // 店铺促销
                    if (null != shopPromotion) {
                        shop.setPromotions(shopPromotion);
                        if (!promotionsIds.contains(shopPromotion.getId())) {
                            promotionsIds.add(shopPromotion.getId());
                            // 店铺满赠
                            if (shopPromotion.getMode() == 3) {
                                // 获取赠品
                                List<CartItemVo> giftList = baseMapper.getPromotionGifts(shopPromotion.getId());
                                shopPromotion.setGiftList(giftList);
                            }
                            // 设置该优惠下的商品列表
                            List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
                            productList.forEach(item -> {
                                if (!(1 == item.getSvipProductFlag() && svipUserFlag)){
                                    // 超级会员购买svip商品不享受店铺促销优惠
                                    PromotionPriceVo promotionPrice = new PromotionPriceVo();
                                    promotionPrice.setProductId(item.getProductId());
                                    promotionPrice.setNumber(item.getNumber());
                                    promotionPrice.setPrice(item.getPrice());
                                    promotionPriceList.add(promotionPrice);
                                }
                            });
                            shopPromotion.setProductList(promotionPriceList);
                            // 添加到总促销
                            totalPromotions.add(shopPromotion);
                            // 店铺满减-计算供应商总价/满减金额
                            if (shopPromotion.getMode() == 2 && MathUtil.compare(shopPromotionFee, shopPromotion.getTouchPrice()) > -1) {
                                // 该供应商总价 - 满减金额
                                shopPrice.set(MathUtil.sub(shopPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                // 该供应商优惠总额 + 满减金额
                                shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                            }
                        }
                    }
                    // 是否可领取优惠券
                    Boolean couponsLogo = setCouponsLogo(shop.getShopId(), source, productList, couponList);
                    shop.setCouponsLogo(couponsLogo);

                    // 供应商商品
                    shop.setCartList(productList);
                    // 供应商总价
                    shop.setTotalPrice(shopPrice.get());
                    // 供应商总优惠
                    shop.setReducedPrice(shopReducedPrice.get());
                    // 供应商总超级会员优惠
                    shop.setSvipReducedPrice(shopSvipReducedPrice.get());
                    // 供应商划线价
                    shop.setOriginalPrice(shopOriginalPrice.get());
                    // 计算总价
                    totalPrice.set(MathUtil.add(totalPrice, shop.getTotalPrice()).doubleValue());
                    // 优惠总额
                    reducedPrice.set(MathUtil.add(reducedPrice, shop.getReducedPrice()).doubleValue());
                    // 总划线价
                    totalOriginalPrice.set(MathUtil.add(totalOriginalPrice, shop.getOriginalPrice()).doubleValue());
                    // 商品种类
                    kindCount.updateAndGet(v -> v + shopKindCount.get());
                }
            });
            // 删除空数据
            shopInfoList.removeIf(shop -> (shop.getCount() == 0));
            // 总促销计算
            totalPromotions.forEach(promotions -> {
                // 凑单满减
                if (promotions.getType() == 2 && promotions.getMode() == 2) {
                    // 该促销内商品总价
                    double touchPrice = promotions.getProductList().stream().mapToDouble(product -> product.getNumber() * product.getPrice()).sum();
                    if (MathUtil.compare(touchPrice, promotions.getTouchPrice()) > -1) {
                        // 总价 - 满减金额
                        totalPrice.set(MathUtil.sub(totalPrice, promotions.getReducedPrice()).doubleValue());
                        // 优惠总额 + 满减金额
                        reducedPrice.set(MathUtil.add(reducedPrice, promotions.getReducedPrice()).doubleValue());
                    }
                }
            });
        }
        // 过滤与当前购物车商品无关的优惠券
        filterCoupon(source, cartAllProducts, couponList);
        /*
         * 返回结果
         */
        Map<String, Object> resultMap = new HashMap<>(8);
        resultMap.put("list", shopInfoList);
        resultMap.put("invalid", invalidList);
        resultMap.put("kindCount", kindCount);
        resultMap.put("totalCount", totalCount);
        resultMap.put("totalPrice", totalPrice);
        resultMap.put("reducedPrice", reducedPrice);
        resultMap.put("totalOriginalPrice", totalOriginalPrice);
        resultMap.put("promotions", totalPromotions);
        resultMap.put("couponList", couponList);
        // 返回数据
        return ResponseJson.success(resultMap);
    }

    /**
     * 过滤与当前商品列表无关的优惠券
     */
    private void filterCoupon(Integer source, List<CartItemVo> cartItems, List<CouponVo> couponList) {
        if (couponList != null && couponList.size() > 0) {
            Iterator<CouponVo> iterator = couponList.iterator();
            A:
            while (iterator.hasNext()) {
                CouponVo coupon = iterator.next();
                if (coupon.getCouponType() == 0 && coupon.getProductType() == 2) {
                    // 活动券商品Ids
                    List<Integer> productIds = orderCommonMapper.getCouponProductIds(coupon.getCouponId(), source);
                    coupon.setProductIds(productIds);
                    for (CartItemVo item : cartItems) {
                        if (!productIds.contains(item.getProductId())) {
                            iterator.remove();
                            continue A;
                        }
                    }
                }
                if (coupon.getCouponType() == 1) {
                    for (CartItemVo item : cartItems) {
                        if (!coupon.getCategoryType().equals(item.getCommodityType())) {
                            iterator.remove();
                            continue A;
                        }
                    }
                }
                if (coupon.getCouponType() == 3) {
                    for (CartItemVo item : cartItems) {
                        if (!coupon.getShopId().equals(item.getShopId())) {
                            iterator.remove();
                            continue A;
                        }
                    }
                }
                if (coupon.getCouponType() == 3 && coupon.getShopId() != null) {
                    //店铺券
                    String shopName = baseMapper.getShopNameById(coupon.getShopId());
                    if (StringUtils.isNotBlank(shopName)) {
                        if (shopName.length() > 7) {
                            shopName = shopName.substring(0, 6) + "...";
                        }
                        coupon.setShopName(shopName);
                    }
                }
            }
        }
    }

    /**
     * 优惠券标识是否显示
     */
    public Boolean setCouponsLogo(Integer shopId, Integer source, List<CartItemVo> cartItems,  List<CouponVo> couponList) {
        boolean couponsLogo = false;
        if (couponList != null && couponList.size() > 0) {
            A:
            for (CouponVo coupon : couponList) {
                if (coupon.getCouponType() == 4 || coupon.getCouponType() == 2) {
                    //用户券与用户专享券
                    couponsLogo = true;
                    break;
                }
                if (coupon.getCouponType() == 0) {
                    if (coupon.getProductType() == 1) {
                        //活动券-全商城商品
                        couponsLogo = true;
                        break;
                    } else {
                        // 活动券商品Ids
                        List<Integer> productIds = orderCommonMapper.getCouponProductIds(coupon.getCouponId(), source);
                        for (CartItemVo item : cartItems) {
                            if (productIds.contains(item.getProductId())) {
                                couponsLogo = true;
                                break A;
                            }
                        }
                    }
                }
                if (coupon.getCouponType() == 3 && shopId.equals(coupon.getShopId())) {
                    //店铺券
                    couponsLogo = true;
                    break;
                }
                if (coupon.getCouponType() == 1) {
                    //品类券
                    for (CartItemVo item : cartItems) {
                        if (coupon.getCategoryType().equals(item.getCommodityType())) {
                            couponsLogo = true;
                            break A;
                        }
                    }
                }
            }
        }
        return couponsLogo;
    }

    /**
     * 网站顶部购物车数据
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getShoppingCartHead(Integer userId) {
        /*
         * 初始化返回数据
         */
        // 商品总数量
        AtomicInteger totalCount = new AtomicInteger(0);
        // 商品种类
        AtomicInteger kindCount = new AtomicInteger(0);
        // 统计商品总金额
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 购物车商品列表
        List<CartItemVo> cartList = null;
        /*
         * 处理数据
         */
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        List<Integer> promotionsIds = new ArrayList<>();
        // 用户身份
        Integer userIdentity = baseMapper.getIdentityByUserId(userId);
        // 获取购物车商品列表（不区分供应商）
        cartList = cartClubMapper.getCartProductList(userId);
        if (null != cartList && cartList.size()>0) {
            // 移除价格不可见商品
            cartList.removeIf(cartItemVo -> !(cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && userIdentity == 2)));
            cartList.forEach(cartItemVo -> {
                // 设置商品图片及税费
                boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                // 获取商品促销信息
                PromotionsVo promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                /*
                 * 设置商品促销优惠
                 */
                if (null != promotions) {
                    // 关闭阶梯价格，活动优先
                    cartItemVo.setLadderFlag(0);
                    // 商品处于活动状态
                    cartItemVo.setActStatus(1);
                    // 当前促销活动的价格计算列表
                    List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                    // 更新到总促销列表
                    productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList);
                } else {
                    if (cartItemVo.getLadderFlag() == 1) {
                        // 设置阶梯价
                        productService.setCartLadderPrices(cartItemVo, taxFlag);
                        // 头部只展示简略信息，不显示详细阶梯价格
                        cartItemVo.setLadderPrices(null);
                    } else {
                        // 复购价
                        Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getProductId(), userId);
                        if (null != repurchase && repurchase > 0) {
                            if (taxFlag) {
                                BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                                cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                            } else {
                                cartItemVo.setPrice(repurchase);
                            }
                        }
                    }
                }
                // 商品总金额累加
                totalPrice.set(MathUtil.add(totalPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                // 商品种类 +1
                kindCount.incrementAndGet();
                // 购物车总数量 + 当前商品购买数量
                totalCount.updateAndGet(v -> v + cartItemVo.getNumber());
            });
            // 总促销-满减计算
            totalPromotions.forEach(promotions -> {
                // 满减
                if (promotions.getMode() == 2) {
                    // 该促销内商品总价
                    double touchPrice = promotions.getProductList().stream().mapToDouble(product -> product.getNumber() * product.getPrice()).sum();
                    if (MathUtil.compare(touchPrice, promotions.getTouchPrice()) > -1) {
                        // 总价 - 满减金额
                        totalPrice.set(MathUtil.sub(totalPrice, promotions.getReducedPrice()).doubleValue());
                    }
                }
            });
        }
        /*
         * 返回结果
         */
        Map<String, Object> resultMap = new HashMap<>(4);
        resultMap.put("list", cartList);
        resultMap.put("kindCount", kindCount);
        resultMap.put("totalCount", totalCount);
        resultMap.put("totalPrice", totalPrice);
        // 返回数据
        return ResponseJson.success(resultMap);
    }

    /**
     * 添加购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     *                }
     */
    @Override
    public ResponseJson<Integer> addShoppingCart(CartDto cartDto) {
        CartPo cart = cartClubMapper.getCartPo(cartDto);
        if (cart != null) {
            // 购物车已存在该商品，更新数量
            cart.setProductCount(cart.getProductCount() + cartDto.getProductCount());
            cart.setAddTime(new Date());
            cartClubMapper.updateCart(cart);
        } else {
            // 添加新购物车
            cart = new CartPo();
            cart.setUserId(cartDto.getUserId());
            cart.setProductId(cartDto.getProductId());
            cart.setProductCount(cartDto.getProductCount());
            Integer shopId = baseMapper.getShopIdByproductId(cartDto.getProductId());
            cart.setShopId(shopId);
            //判断是否是再次购买
            Double repurchase = baseMapper.getRepurchasePrice(cartDto.getProductId(), cartDto.getUserId());
            if (null != repurchase && repurchase > 0) {
                // 再次购买
                cart.setReBuyFlag(1);
            } else {
                cart.setReBuyFlag(0);
            }
            cart.setAddTime(new Date());
            cartClubMapper.insertCart(cart);
        }
        // 获取购物车数量（商品种类数）
        int cartCount = getCartCount(cartDto.getUserId());
        return ResponseJson.success("添加成功！返回购物车数量", cartCount);
    }

    /**
     * 更新购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     *                }
     */
    @Override
    public ResponseJson<Integer> updateShoppingCart(CartDto cartDto) {
        CartPo cart = new CartPo();
        cart.setUserId(cartDto.getUserId());
        cart.setProductId(cartDto.getProductId());
        cart.setProductCount(cartDto.getProductCount());
        cart.setAddTime(new Date());
        cartClubMapper.updateCart(cart);
        // 获取购物车数量（商品种类数）
        int cartCount = getCartCount(cartDto.getUserId());
        return ResponseJson.success("更新成功！返回购物车数量", cartCount);
    }

    /**
     * 删除购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productIds   商品ids,逗号隔开
     *                }
     */
    @Override
    public ResponseJson<Integer> deleteShoppingCart(CartDto cartDto) {
        List<String> productIdList = Lists.newArrayList();
        if (cartDto.getProductIds().contains(",")) {
            String[] productArr = cartDto.getProductIds().split(",");
            Collections.addAll(productIdList, productArr);
        } else {
            productIdList.add(cartDto.getProductIds());
        }
        cartClubMapper.deleteCartByProductIds(cartDto.getUserId(), productIdList);
        // 获取购物车数量（商品种类数）
        int cartCount = getCartCount(cartDto.getUserId());
        return ResponseJson.success("删除成功！返回购物车数量", cartCount);
    }

    /**
     * 获取机构购物车数量（商品种类数）
     * @param userId 用户ID
     * @return int
     */
    private int getCartCount(Integer userId){
        if (null == userId) {
            return 0;
        }
        // 获取购物车商品列表（不区分供应商）
        List<CartItemVo> cartList = cartClubMapper.getCartProductList(userId);
        if (null != cartList && cartList.size()>0) {
            // 用户身份
            Integer userIdentity = baseMapper.getIdentityByUserId(userId);
            // 移除价格不可见商品
            cartList.removeIf(cartItemVo -> !(cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && userIdentity == 2)));
            return cartList.size();
        } else {
            return 0;
        }
    }


    /**
     * 购物车结算
     *
     * @param userId     用户ID
     * @param productIds 商品ids,逗号隔开
     * @param source     来源 : 1 网站 ; 2 小程序
     */
    @Override
    public ResponseJson<Map<String, Object>> settlementShoppingCart(Integer userId, String productIds, Integer source) {
        log.info("￥￥￥￥￥￥￥￥￥￥ > 商品结算");
        // 商品种类
        AtomicInteger kindCount = new AtomicInteger(0);
        // 商品总数量
        AtomicInteger totalCount = new AtomicInteger(0);
        // 统计商品总金额
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 统计总促销满减
        AtomicDouble reducedPrice = new AtomicDouble(0);
        // 统计总超级会员优惠
        AtomicDouble svipReducedPrice = new AtomicDouble(0);
        // 统计总划线价
        AtomicDouble totalOriginalPrice = new AtomicDouble(0);
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        // 是否包含充值商品，默认false
        AtomicBoolean includeRecharge = new AtomicBoolean(false);
        //所有的商品数据
        List<CartItemVo> totalProductList = new ArrayList<>();
        // 促销活动Id集合，用于合并促销活动
        List<Integer> promotionsIds = new ArrayList<>();
        // 供应商Id集合，用户判断赠品供应商是否在当前供应商中
        List<Integer> shopIds = new ArrayList<>();
        // 用户身份
        Integer userIdentity = baseMapper.getIdentityByUserId(userId);
        // 超级会员标识
        Integer svipUserId = baseMapper.getSvipUserIdByUserId(userId);
        boolean svipUserFlag = null != svipUserId;

        // 前端接收商品Id信息
        List<String> productIdList = new ArrayList<>();
        if (productIds.contains(",")) {
            productIdList = Stream.of(productIds.split(",")).collect(Collectors.toList());
        } else {
            productIdList.add(productIds);
        }

        // 供应商列表
        List<CartShopVo> shopList = cartClubMapper.getCartShopsByProductIds(userId, productIdList);
        if (null != shopList && shopList.size()>0) {
            // 遍历供应商列表
            List<String> finalIdList = productIdList;
            shopList.forEach(shop -> {
                if (null != shop) {
                    // 该供应商下商品种类
                    AtomicInteger shopKindCount = new AtomicInteger(0);
                    // 该供应商总价
                    AtomicDouble shopPrice = new AtomicDouble(0);
                    // 该供应商下参与店铺促销的商品总价
                    AtomicDouble shopPromotionFee = new AtomicDouble(0);
                    // 该供应商满减金额（供应商满减，单品满减）
                    AtomicDouble shopReducedPrice = new AtomicDouble(0);
                    // 该供应商svip优惠金额
                    AtomicDouble shopSvipReducedPrice = new AtomicDouble(0);
                    // 该供应商划线价
                    AtomicDouble shopOriginalPrice = new AtomicDouble(0);
                    // 供应商促销优惠活动
                    PromotionsVo shopPromotion = baseMapper.getPromotionByShopId(shop.getShopId());
                    // 供应商下商品列表
                    List<CartItemVo> productList = cartClubMapper.getCartProductsByShopIdAndProductIds(userId, shop.getShopId(), finalIdList);
                    if (null != productList && productList.size()>0) {
                        // 去除价格不可见商品
                        productList.removeIf(cartItemVo -> !(cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && userIdentity == 2)));
                        // 去除库存不足商品
                        productList.removeIf(cartItemVo -> !(cartItemVo.getStock() != null && cartItemVo.getStock() > 0 && cartItemVo.getStock() >= cartItemVo.getMin() && cartItemVo.getStock() >= cartItemVo.getNumber()));
                        // 迭代器设置商品信息
                        for (CartItemVo cartItemVo : productList) {
                            // 设置商品图片及税费
                            boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                            // 是否充值商品
                            boolean recharge = productService.isRechargeProduct(cartItemVo.getProductId());
                            if (recharge) {
                                includeRecharge.set(true);
                            }
                            if (1 == cartItemVo.getSvipProductFlag()) {
                                // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                                // 超级会员设置商品优惠价
                                productService.setSvipProductPrice(cartItemVo, taxFlag, svipUserFlag);
                                if (svipUserFlag) {
                                    // 超级会员设置超级会员优惠总额 + 商品优惠金额
                                    shopSvipReducedPrice.set(MathUtil.add(shopSvipReducedPrice, cartItemVo.getSvipTotalReducedPrice()).doubleValue());
                                }
                            } else {
                                // 获取商品促销信息
                                PromotionsVo promotions = null;
                                // 没有店铺促销时，商品促销才有效
                                if (null == shopPromotion) {
                                    // 获取商品促销信息
                                    promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                                    /*
                                     * 设置商品促销优惠
                                     */
                                    if (null != promotions) {
                                        // 当前促销活动的价格计算列表
                                        List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                                        // 更新到总促销列表
                                        productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList);
                                        //单品满减-计算供应商总价/满减金额
                                        if (promotions.getType() == 1 && promotions.getMode() == 2) {
                                            BigDecimal totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice());
                                            if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) > -1) {
                                                // 如果满足促销条件，设置供应商价格-满减金额，满减总额 + 当前促销满减金额
                                                shopPrice.set(MathUtil.sub(shopPrice.get(), promotions.getReducedPrice()).doubleValue());
                                                shopReducedPrice.set(MathUtil.add(shopReducedPrice, promotions.getReducedPrice()).doubleValue());
                                            }
                                        }
                                        cartItemVo.setPromotions(promotions);
                                    }
                                }
                                if (null != promotions || null != shopPromotion) {
                                    // 商品处于活动状态
                                    cartItemVo.setActStatus(1);
                                    // 关闭阶梯价格，活动优先
                                    cartItemVo.setLadderFlag(0);
                                } else {
                                    if (cartItemVo.getLadderFlag() == 1) {
                                        // 设置阶梯价
                                        productService.setCartLadderPrices(cartItemVo, taxFlag);
                                    } else {
                                        // 复购价
                                        Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getProductId(), userId);
                                        if (null != repurchase && repurchase > 0) {
                                            if (taxFlag) {
                                                BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                                                cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                                            } else {
                                                cartItemVo.setPrice(repurchase);
                                            }
                                        }
                                    }
                                }
                            }
                            // 该供应商下价格累加
                            shopPrice.set(MathUtil.add(shopPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                            shopOriginalPrice.set(MathUtil.add(shopOriginalPrice.get(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getOriginalPrice())).doubleValue());
                            // 该供应商下可参与店铺促销的商品价格累加
                            if (!(1 == cartItemVo.getSvipProductFlag() && svipUserFlag)) {
                                // 超级会员购买svip商品不享受店铺促销优惠
                                shopPromotionFee.set(MathUtil.add(shopPromotionFee, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                            }
                            // 该供应商下商品种类 +1
                            shopKindCount.incrementAndGet();
                            // 总数量 + 当前商品购买数量
                            totalCount.updateAndGet(v -> v + cartItemVo.getNumber());
                        }
                        if (shopKindCount.get() > 0) {
                            // 店铺促销
                            if (null != shopPromotion) {
                                shop.setPromotions(shopPromotion);
                                if (!promotionsIds.contains(shopPromotion.getId())) {
                                    promotionsIds.add(shopPromotion.getId());
                                    // 店铺满赠
                                    if (shopPromotion.getMode() == 3) {
                                        // 获取赠品
                                        List<CartItemVo> giftList = baseMapper.getPromotionGifts(shopPromotion.getId());
                                        shopPromotion.setGiftList(giftList);
                                    }
                                    // 设置该优惠下的商品列表
                                    List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
                                    productList.forEach(item -> {
                                        if (!(1 == item.getSvipProductFlag() && svipUserFlag)){
                                            // 超级会员购买svip商品不享受店铺促销优惠
                                            PromotionPriceVo promotionPrice = new PromotionPriceVo();
                                            promotionPrice.setProductId(item.getProductId());
                                            promotionPrice.setNumber(item.getNumber());
                                            promotionPrice.setPrice(item.getPrice());
                                            promotionPriceList.add(promotionPrice);
                                        }
                                    });
                                    shopPromotion.setProductList(promotionPriceList);
                                    // 添加到总促销
                                    totalPromotions.add(shopPromotion);
                                    // 店铺满减-计算供应商总价/满减金额
                                    if (shopPromotion.getMode() == 2 && MathUtil.compare(shopPromotionFee, shopPromotion.getTouchPrice()) > -1) {
                                        // 该供应商总价 - 满减金额
                                        shopPrice.set(MathUtil.sub(shopPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                        // 该供应商优惠总额 + 满减金额
                                        shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                    }
                                }
                            }
                            // 供应商商品
                            shop.setCartList(productList);
                            // 供应商总价
                            shop.setTotalPrice(shopPrice.get());
                            // 供应商总优惠
                            shop.setReducedPrice(shopReducedPrice.get());
                            // 供应商总超级会员优惠
                            shop.setSvipReducedPrice(shopSvipReducedPrice.get());
                            // 供应商划线价
                            shop.setOriginalPrice(shopOriginalPrice.get());
                            // 添加供应商Id集合
                            shopIds.add(shop.getShopId());
                            // 计算总价
                            totalPrice.set(MathUtil.add(totalPrice, shop.getTotalPrice()).doubleValue());
                            // 优惠总额
                            reducedPrice.set(MathUtil.add(reducedPrice, shop.getReducedPrice()).doubleValue());
                            // 超级会员优惠总额
                            svipReducedPrice.set(MathUtil.add(svipReducedPrice, shop.getSvipReducedPrice()).doubleValue());
                            // 总划线价
                            totalOriginalPrice.set(MathUtil.add(totalOriginalPrice, shop.getOriginalPrice()).doubleValue());
                            // 商品种类
                            kindCount.updateAndGet(v -> v + shopKindCount.get());
                        }
                        totalProductList.addAll(productList);
                    }
                    // 供应商下商品种类
                    shop.setCount(shopKindCount.get());
                }
            });
            // 删除空数据
            shopList.removeIf(shop -> (null == shop || shop.getCount() == 0));

        }
        // 总促销计算
        totalPromotions.forEach(promotions -> {
            // 该促销内商品总价
            double touchPrice = promotions.getProductList().stream().mapToDouble(product -> product.getNumber() * product.getPrice()).sum();
            // 满足促销条件
            if (MathUtil.compare(touchPrice, promotions.getTouchPrice()) > -1) {
                // 凑单满减
                if (promotions.getType() == 2 && promotions.getMode() == 2) {
                    // 总价 - 满减金额
                    totalPrice.set(MathUtil.sub(totalPrice, promotions.getReducedPrice()).doubleValue());
                    // 优惠总额 + 满减金额
                    reducedPrice.set(MathUtil.add(reducedPrice, promotions.getReducedPrice()).doubleValue());
                } else if (promotions.getMode() == 3) {
                    // 全部满赠
                    promotions.getGiftList().forEach(gift -> {
                        if (shopIds.contains(gift.getShopId())) {
                            // 赠品在当前订单内的供应商下
                            shopList.forEach(shop -> {
                                if (shop.getShopId().equals(gift.getShopId())) {
                                    shop.getCartList().add(gift);
                                }
                            });
                        } else {
                            // 获取赠品供应商
                            CartShopVo giftShop = baseMapper.getShopByProductId(gift.getProductId());
                            shopIds.add(giftShop.getShopId());
                            giftShop.setCartList(new ArrayList<>());
                            giftShop.getCartList().add(gift);
                            shopList.add(giftShop);
                        }
                    });
                }
            }
        });

        // 发票信息
        InvoiceVo invoice = baseMapper.getUserInvoice(userId);
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(userId);
        // 可用采美豆
        Integer userBeans = baseMapper.getUserBeans(userId);
        // 用户可用优惠券（总）
        List<CouponVo> couponList = getUserCartCoupons(totalProductList, source, userId);

        // 包装返回数据
        Map<String, Object> resultData = new HashMap<>();
        resultData.put("list", shopList);
        resultData.put("kindCount", kindCount);
        resultData.put("totalCount", totalCount);
        resultData.put("totalPrice", totalPrice);
        resultData.put("reducedPrice", reducedPrice);
        resultData.put("svipReducedPrice", svipReducedPrice);
        resultData.put("totalOriginalPrice", totalOriginalPrice);
        resultData.put("promotions", totalPromotions);
        resultData.put("invoice", invoice);
        resultData.put("userMoney", availableMoney);
        resultData.put("userBeans", userBeans);
        resultData.put("includeRecharge", includeRecharge);
        resultData.put("couponList", couponList);
        return ResponseJson.success(resultData);
    }

    /**
     * 立即购买
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     *                source       来源 : 1 网站 ; 2 小程序
     *                }
     */
    @Override
    public ResponseJson<Map<String, Object>> buyNowProduct(CartDto cartDto) {
        log.info("￥￥￥￥￥￥￥￥￥￥ > 立即购买");
        // 供应商信息
        CartShopVo shop = baseMapper.getShopByProductId(cartDto.getProductId());
        if (null == shop) {
            return ResponseJson.error("商品供应商异常！", null);
        }
        CartItemVo cartItemVo = cartClubMapper.getCartItemByProductId(cartDto.getProductId());
        if (null == cartItemVo) {
            return ResponseJson.error("商品数据异常！", null);
        }
        // 用户身份
        Integer userIdentity = baseMapper.getIdentityByUserId(cartDto.getUserId());
        // 价格不可见商品
        if (!(cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && userIdentity == 2))) {
            return ResponseJson.error("商品价格不可见！", null);
        }
        cartItemVo.setNumber(cartDto.getProductCount());
        // 库存不足商品
        if(!(cartItemVo.getStock() != null && cartItemVo.getStock() > 0 && cartItemVo.getStock() >= cartItemVo.getMin() && cartItemVo.getStock() >= cartItemVo.getNumber())){
            return ResponseJson.error("商品库存不足！", null);
        }
        List<CartShopVo> shopList = new ArrayList<>();
        // 设置商品图片及税费
        boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        // 统计商品总金额
        AtomicDouble originalPrice = new AtomicDouble(0);
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 统计总促销满减
        AtomicDouble reducedPrice = new AtomicDouble(0);

        // 供应商促销优惠活动
        PromotionsVo promotions = baseMapper.getPromotionByShopId(shop.getShopId());
        // 没有店铺促销时，商品促销才有效
        if (null == promotions) {
            // 获取商品促销信息
            promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
        } else {
            shop.setPromotions(promotions);
        }
        /*
         * 设置商品促销优惠
         */
        if (null != promotions) {
            // 当前促销活动的价格计算列表
            List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
            promotions.setProductList(promotionPriceList);
            Double totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice()).doubleValue();
            // 满足促销条件
            if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) >= 0) {
                // 满减
                if (promotions.getMode() == 2) {
                    // 总价 - 满减金额
                    totalPrice.set(MathUtil.sub(totalPrice.get(), promotions.getReducedPrice()).doubleValue());
                    // 优惠总额 + 满减金额
                    reducedPrice.set(MathUtil.add(reducedPrice.get(), promotions.getReducedPrice()).doubleValue());
                } else if (promotions.getMode() == 3) {
                    // 满赠
                    List<CartItemVo> giftList = baseMapper.getPromotionGifts(promotions.getId());
                    giftList.forEach(gift -> {
                        if (shop.getShopId().equals(gift.getShopId())) {
                            // 赠品在当前订单内的供应商下
                            if (shop.getShopId().equals(gift.getShopId())) {
                                shop.getCartList().add(gift);
                            }
                        } else {
                            // 获取赠品供应商
                            CartShopVo giftShop = baseMapper.getShopByProductId(gift.getProductId());
                            giftShop.setCartList(new ArrayList<>());
                            giftShop.getCartList().add(gift);
                            shopList.add(giftShop);
                        }
                    });
                    promotions.setGiftList(giftList);
                }
            }
            // 添加到总促销
            totalPromotions.add(promotions);
            // 商品处于活动状态
            cartItemVo.setActStatus(1);
            // 关闭阶梯价格，活动优先
            cartItemVo.setLadderFlag(0);
            cartItemVo.setPromotions(promotions);
        } else {
            cartItemVo.setActStatus(0);
            if (cartItemVo.getLadderFlag() == 1) {
                // 设置阶梯价
                productService.setCartLadderPrices(cartItemVo, taxFlag);
            } else {
                // 复购价
                Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getProductId(), cartDto.getUserId());
                if (null != repurchase && repurchase > 0) {
                    if (taxFlag) {
                        BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                        cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                    } else {
                        cartItemVo.setPrice(repurchase);
                    }
                }
            }
        }
        // 商品总金额累加
        originalPrice.set(MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getOriginalPrice()).doubleValue());
        totalPrice.set(MathUtil.add(totalPrice.get(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
        // 供应商商品
        List<CartItemVo> cartList = new ArrayList<>(1);
        cartList.add(cartItemVo);
        shop.setCartList(cartList);
        shop.setCount(1);
        // 供应商总价
        shop.setTotalPrice(totalPrice.get());
        // 供应商总优惠
        shop.setReducedPrice(reducedPrice.get());
        // 供应商划线价
        shop.setOriginalPrice(originalPrice.get());
        // 当前供应商添加到首位
        shopList.add(0, shop);

        // 是否充值商品
        boolean includeRecharge = false;
        boolean recharge = productService.isRechargeProduct(cartDto.getProductId());
        if (recharge) {
            includeRecharge = true;
        }
        // 发票信息
        InvoiceVo invoice = baseMapper.getUserInvoice(cartDto.getUserId());
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(cartDto.getUserId());
        // 可用采美豆
        Integer userBeans = baseMapper.getUserBeans(cartDto.getUserId());
        // 获取用户可用优惠券
        List<CouponVo> couponList = getUserCartCoupons(cartList, cartDto.getSource(), cartDto.getUserId());
        // 包装返回数据
        Map<String, Object> resultData = new HashMap<>();
        resultData.put("list", shopList);
        resultData.put("kindCount", 1);
        resultData.put("totalCount", cartDto.getProductCount());
        resultData.put("totalPrice", totalPrice.get());
        resultData.put("reducedPrice", reducedPrice.get());
        resultData.put("totalOriginalPrice", originalPrice.get());
        resultData.put("invoice", invoice);
        resultData.put("userMoney", availableMoney);
        resultData.put("userBeans", userBeans);
        resultData.put("includeRecharge", includeRecharge);
        resultData.put("promotions", totalPromotions);
        resultData.put("couponList", couponList);
        return ResponseJson.success(resultData);
    }

    private List<CouponVo> getUserCartCoupons(List<CartItemVo> cartList, Integer source, Integer userId) {
        // 用户可用优惠券（总）
        List<CouponVo> couponList = orderCommonMapper.getClubCouponList(userId);
        Iterator<CouponVo> iterator = couponList.iterator();
        while (iterator.hasNext()) {
            CouponVo coupon = iterator.next();
            BigDecimal totalAmount = BigDecimal.ZERO;
            Integer couponType = coupon.getCouponType();
            for (CartItemVo item : cartList) {
                if (couponType == 0) {
                    if (coupon.getProductType() == 1) {
                        //活动券-全商城商品
                        BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                        totalAmount = MathUtil.add(price, totalAmount);
                    } else if (coupon.getProductType() == 2) {
                        //活动券-指定商品
                        List<Integer> couponProductIds = orderCommonMapper.getCouponProductIds(coupon.getCouponId(), source);
                        if (couponProductIds.contains(item.getProductId())) {
                            BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                            totalAmount = MathUtil.add(price, totalAmount);
                        }
                    }
                } else if (couponType == 1) {
                    //品类券
                    if (coupon.getCategoryType().equals(item.getCommodityType())) {
                        BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                        totalAmount = MathUtil.add(price, totalAmount);
                    }
                } else if (couponType == 2 || couponType == 4) {
                    //用户专享券或新用户券
                    BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                    totalAmount = MathUtil.add(price, totalAmount);
                } else if (coupon.getCouponType() == 3 && coupon.getShopId() != null) {
                    //店铺券
                    String shopName = baseMapper.getShopNameById(coupon.getShopId());
                    if (StringUtils.isNotBlank(shopName)) {
                        if (shopName.length() > 7) {
                            shopName = shopName.substring(0, 6) + "...";
                        }
                        coupon.setShopName(shopName);
                    }
                    if (coupon.getShopId().equals(item.getShopId())) {
                        BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                        totalAmount = MathUtil.add(price, totalAmount);
                    }
                }
            }
            if (MathUtil.compare(totalAmount, coupon.getTouchPrice()) < 0) {
                iterator.remove();
            }
        }
        couponList.sort((o1, o2) -> o2.getCouponAmount().compareTo(o1.getCouponAmount()));
        return couponList;
    }

    /**
     * 获取结算商品运费
     *
     * @param userId     用户ID
     * @param productIds 商品ids,逗号隔开
     * @param townId     地区Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getProductsPostage(Integer userId, String productIds, Integer townId) {
        // 商品Id信息
        List<String> productIdList = new ArrayList<>();
        if (productIds.contains(",")) {
            productIdList = Stream.of(productIds.split(",")).collect(Collectors.toList());
        } else {
            productIdList.add(productIds);
        }
        // 计算运费
        Map<String, Object> resultMap = productService.computePostage(userId, townId, productIdList);
        return ResponseJson.success(resultMap);
    }

    /**
     * 获取用户发票信息
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<InvoiceVo> getUserInvoice(Integer userId) {
        InvoiceVo invoice = cartClubMapper.getUserInvoice(userId);
        return ResponseJson.success(invoice);
    }

    /**
     * 更新用户发票
     *
     * @param invoiceDto {
     *                   userId:用户Id
     *                   id:id
     *                   invoiceTitle:单位名，发票抬头
     *                   corporationTaxNum: 企业税号、纳税人识别号
     *                   registeredAddress:注册地址
     *                   registeredPhone:注册电话
     *                   bankAccountNo:开户银行账户
     *                   openBank:开户银行
     *                   }
     */
    @Override
    public ResponseJson<Integer> updateUserInvoice(InvoiceDto invoiceDto) {
        if (null == invoiceDto.getId()) {
            // 只保留一份用户发票信息
            cartClubMapper.deleteUserInvoice(invoiceDto.getUserId());
            cartClubMapper.insertUserInvoice(invoiceDto);
        } else {
            cartClubMapper.updateUserInvoice(invoiceDto);
        }
        return ResponseJson.success(1);
    }

    /**
     * 供应商优惠券
     *
     * @param userId 用户id
     * @param shopId 供应商id
     * @param source 来源 : 1 网站 ; 2 小程序
     * @param status 状态: 1未领取 2已领取
     */
    @Override
    public ResponseJson<Map<String, Object>> getShopCoupons(Integer userId, Integer shopId, Integer source, Integer status) {
        // 供应商下购物车商品列表
        List<CartItemVo> productList = cartClubMapper.getCartProductsByShopId(shopId, userId);
        // 未领取的优惠券
        List<CouponVo> preCouponList = orderCommonMapper.getPrevCouponList(userId);
        // 过滤与当前购物车商品无关的优惠券
        filterCoupon(source, productList, preCouponList);
        // 用户可用优惠券（已领取）
        List<CouponVo> couponList = orderCommonMapper.getClubCouponList(userId);
        // 过滤与当前购物车商品无关的优惠券
        filterCoupon(source, productList, couponList);
        Map<String, Object> map = new HashMap<>(3);
        map.put("notCouponNum", preCouponList == null ? 0 : preCouponList.size());
        map.put("couponNum", couponList == null ? 0 : couponList.size());
        if (status == 1) {
            //未领取
            map.put("couponList", preCouponList);
        } else {
            //已领取
            map.put("couponList", couponList);
        }
        return ResponseJson.success(map);
    }


}
